import { ColumnOptions, ColumnResolvable, DataTypes, SqliteDataTypes } from "../typings";
import Database from "./Database";
import { Table } from "./Table";
/**
 * Creates a column class for a table.
 */
declare class Column implements ColumnOptions {
    /**
     * The name of the table.
     */
    name: string | null;
    /**
     * The default value for this table.
     */
    default?: any;
    /**
     * Whether this column was be nullable in the database.
     */
    notNull?: boolean | undefined;
    /**
     * Whether this column is primary key to table.
     */
    primary?: boolean | undefined;
    /**
     * The type data for this column
     */
    type: DataTypes | null;
    /**
     * The default cached value.
     */
    _default?: any;
    /**
     * The database this column belongs to.
     */
    db?: Database;
    /**
     *
     * @param data Data for the column.
     */
    constructor(data?: ColumnOptions);
    /**
     * Transforms a column object into a class column.
     * @param data The data for the column.
     * @param db The database this column belongs to.
     * @returns The created column.
     */
    static transform(data: ColumnResolvable, db: Database): Column;
    /**
     * Transforms a value into a valid one for sqlite.
     * @param data The value to transform.
     * @param type The type it has in sqlite.
     * @returns The converted value.
     */
    static transformValue(data: any, type: DataTypes): any;
    /**
     * Attaches a database to this column.
     * @param db The database to attach.
     * @returns The column class.
     */
    attachDatabase(db: Database): this;
    /**
     * Gets the real type for this column.
     * @returns The type it has in the database.
     */
    getRealType(): SqliteDataTypes;
    /**
     * Whether this column is attached to a database
     * @returns
     */
    hasDatabase(): this is this & {
        db: Database;
    };
    /**
     * Creates this column in given table.
     * @param table The table to create this column in.
     * @returns The column.
     */
    create(table: Table): this;
    /**
     * Gets a string to create this column under a table.
     * @returns The string.
     */
    get creator(): string;
    /**
     * Sets a name for this column.
     * @param str The name for the column.
     * @returns The column class.
     */
    setName(str: string): this;
    /**
     * Sets a data type for this column.
     * @param type The type for this column.
     * @returns The column class.
     */
    setType(type: DataTypes): this;
    /**
     * Sets the default value for this column.
     * @param value The value for the column marked as default.
     * @returns The column class.
     */
    setDefault(value: any): this;
    /**
     * Gets the default value for this column.
     */
    get defaultValue(): any;
    /**
     * Sets whether the column can be nullable or not.
     * @param bool
     * @returns The column class.
     */
    setNotNull(bool: boolean): this;
    /**
     * Sets whether this column is primary key to table.
     * @param bool Boolean indicating if it is the primary column.
     * @returns The column class.
     */
    setPrimary(bool: boolean): this;
}
export { Column };
//# sourceMappingURL=Column.d.ts.map