"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Column = void 0;
const util_1 = require("util");
const Util_1 = require("../util/Util");
const DatabaseError_1 = require("./DatabaseError");
/**
 * Creates a column class for a table.
 */
class Column {
    /**
     *
     * @param data Data for the column.
     */
    constructor(data) {
        this.name = null;
        this.type = null;
        if (data)
            for (const key of Object.keys(data))
                this[key] = data[key];
    }
    /**
     * Transforms a column object into a class column.
     * @param data The data for the column.
     * @param db The database this column belongs to.
     * @returns The created column.
     */
    static transform(data, db) {
        return data instanceof Column ? data.attachDatabase(db) : new Column(data).attachDatabase(db);
    }
    /**
     * Transforms a value into a valid one for sqlite.
     * @param data The value to transform.
     * @param type The type it has in sqlite.
     * @returns The converted value.
     */
    static transformValue(data, type) {
        if (type === "BLOB")
            return data;
        else if (type === "INTEGER" || type === "REAL") {
            if (!Util_1.Util.isNumber(data)) {
                throw new DatabaseError_1.DatabaseError(`INVALID NUMBER: ${data}`);
            }
            return typeof data === "string" ? Number(data) : data;
        }
        else if (type === "NUMERIC") {
            if (!Util_1.Util.isNumber(data)) {
                throw new DatabaseError_1.DatabaseError(`INVALID NUMERIC NUMBER: ${data}`);
            }
            return typeof data === "string" ? Number(data) : data;
        }
        else if (type === "TEXT") {
            return `'${data === null || data === void 0 ? void 0 : data.toString().replace(/'/g, "")}'`;
        }
        else if (type === "JSON") {
            if (typeof data !== "object" || (typeof data === "object" && data === null)) {
                throw new DatabaseError_1.DatabaseError("INVALID JSON: " + (0, util_1.inspect)(data));
            }
            try {
                var json = JSON.stringify(data);
                return `'${json.toString().replace(/'/g, "")}'`;
            }
            catch (error) {
                throw new DatabaseError_1.DatabaseError(`INVALID JSON: ${(0, util_1.inspect)(data)}`);
            }
        }
        else if (type === "BOOLEAN") {
            if (typeof data !== "boolean") {
                throw new DatabaseError_1.DatabaseError("INVALID BOOLEAN: " + data);
            }
            return data ? 1 : 0;
        }
    }
    /**
     * Attaches a database to this column.
     * @param db The database to attach.
     * @returns The column class.
     */
    attachDatabase(db) {
        this.db = db;
        return this;
    }
    /**
     * Gets the real type for this column.
     * @returns The type it has in the database.
     */
    getRealType() {
        if (!this.type)
            throw new DatabaseError_1.DatabaseError("NO TYPE COLUMN: " + this.name);
        return this.type === "JSON" ? "TEXT" : this.type === "BOOLEAN" ? "INTEGER" : this.type;
    }
    /**
     * Whether this column is attached to a database
     * @returns
     */
    hasDatabase() {
        if (!this.db) {
            throw new DatabaseError_1.DatabaseError(`No database available for column ${this.name}`);
        }
        return Boolean(this.db);
    }
    /**
     * Creates this column in given table.
     * @param table The table to create this column in.
     * @returns The column.
     */
    create(table) {
        var _a;
        if (this.hasDatabase()) {
            (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`
                    ALTER TABLE
                        ${table.name}
                    ADD COLUMN 
                        ${this.creator}
                `).run();
        }
        return this;
    }
    /**
     * Gets a string to create this column under a table.
     * @returns The string.
     */
    get creator() {
        const data = [
            this.name,
            this.getRealType()
        ];
        if (this.primary)
            data.push(`PRIMARY KEY`);
        if (this.notNull)
            data.push(`NOT NULL`);
        if (this.defaultValue)
            data.push(`DEFAULT ${this.defaultValue}`);
        return data.join(" ");
    }
    /**
     * Sets a name for this column.
     * @param str The name for the column.
     * @returns The column class.
     */
    setName(str) {
        this.name = str;
        return this;
    }
    /**
     * Sets a data type for this column.
     * @param type The type for this column.
     * @returns The column class.
     */
    setType(type) {
        this.type = type;
        return this;
    }
    /**
     * Sets the default value for this column.
     * @param value The value for the column marked as default.
     * @returns The column class.
     */
    setDefault(value) {
        if (this.type)
            this._default = Column.transformValue(value, this.type);
        this.default = value;
        return this;
    }
    /**
     * Gets the default value for this column.
     */
    get defaultValue() {
        var _a;
        return this.type && this.default !== undefined ? (_a = this._default) !== null && _a !== void 0 ? _a : (() => {
            const val = Column.transformValue(this.default, this.type);
            this._default = val;
            return val;
        })() : null;
    }
    /**
     * Sets whether the column can be nullable or not.
     * @param bool
     * @returns The column class.
     */
    setNotNull(bool) {
        this.notNull = bool;
        return this;
    }
    /**
     * Sets whether this column is primary key to table.
     * @param bool Boolean indicating if it is the primary column.
     * @returns The column class.
     */
    setPrimary(bool) {
        this.primary = bool;
        return this;
    }
}
exports.Column = Column;
//# sourceMappingURL=Column.js.map