"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tiny_typed_emitter_1 = require("tiny-typed-emitter");
const better_sqlite3_1 = __importDefault(require("better-sqlite3"));
const constants_1 = require("../util/constants");
const Util_1 = require("../util/Util");
const Table_1 = require("./Table");
const DatabaseError_1 = require("./DatabaseError");
/**
 * Database wrapper class for sqlite.
 */
class Database extends tiny_typed_emitter_1.TypedEmitter {
    /**
     *
     * @param options The options for this database.
     */
    constructor(options = constants_1.DefaultDatabaseOptions) {
        super();
        /**
         * The assigned tables for this database.
         */
        this.tables = new Map();
        this.options = this._validateOptions(Util_1.Util.mergeDefault(options, constants_1.DefaultDatabaseOptions));
    }
    /**
     * Validates given options.
     * @param options The options to check.
     * @returns The options with their default values.
     */
    _validateOptions(options) {
        return options;
    }
    /**
     * Resolves a string or object into a table.
     * @param name The name of the table.
     * @param suppress Whether to suppress the error if table does not exist.
     * @returns The table found.
     */
    resolveTable(name, suppress = false) {
        var _a;
        let table = null;
        if (name instanceof Table_1.Table)
            table = name;
        if (typeof name === "string")
            table = (_a = this.tables.get(name)) !== null && _a !== void 0 ? _a : null;
        if (!table && !suppress)
            throw new DatabaseError_1.DatabaseError(`Invalid table ${name}.`);
        return table;
    }
    /**
     * Deletes garbage from this database.
     * @returns
     */
    vacuum() {
        var _a;
        return (_a = this.db) === null || _a === void 0 ? void 0 : _a.pragma("VACUUM");
    }
    /**
     * Creates a table.
     * @param name The name for this table
     * @returns The table that was created.
     */
    createTable(name) {
        const exists = this.resolveTable(name, true);
        if (exists)
            return exists;
        const table = new Table_1.Table(name, this);
        this.tables.set(table.name, table);
        return table;
    }
    /**
     * Adds a table to this database.
     * @param table The table to add.
     * @returns The database this table was added to.
     */
    addTable(table) {
        this.tables.set(table.name, table);
        return this;
    }
    /**
     * Returns the corresponding array from given map.
     * @param map The map to convert.
     * @returns The array from the map.
     */
    arrayFromMap(map) {
        return [...this.tables.values()];
    }
    get readyTimestamp() {
        return this.readyAt ? this.readyAt.getTime() : undefined;
    }
    /**
     * Gets data from a table.
     * @param name The name of the table to get its data.
     * @returns The table data if it exists.
     */
    getTableData(name) {
        var _a;
        try {
            const data = (_a = this.db) === null || _a === void 0 ? void 0 : _a.prepare(`SELECT * FROM sqlite_master WHERE type = ? AND name = ?`).get("table", name);
            return data;
        }
        catch (error) {
            console.log(error);
            return undefined;
        }
    }
    /**
     * Gets all available tables from this database.
     * @returns An array of table data.
     */
    get databaseTables() {
        var _a;
        return (_a = this.db) === null || _a === void 0 ? void 0 : _a.prepare(`
        SELECT 
            *
        FROM 
            sqlite_master 
        WHERE 
            type ='table'
        AND 
            name NOT LIKE 'sqlite_%'`).all();
    }
    /**
     * Gets multiple rows from this database.
     * @param table The table to get rows from.
     * @param options The options to apply for this query.
     * @returns The rows found from this query.
     */
    all(table, options) {
        return this.resolveTable(table).all(options);
    }
    /**
     * Gets the position of a row in a table.
     * @param table The table to get row position from.
     * @param sortOptions Options for sorting this query.
     * @param whereOptions Options to filter this query.
     * @returns The position or positions of rows (and possibly their data if multiple).
     * @example
     * const posOf = db.get("mytable", {
     *      columns: ["bank"]
     * }, {
     *      where: {
     *          column: "id",
     *          equals: "123456"
     *      }
     * })
     *
     * console.log(posOf.position)
     */
    getRowPosition(table, sortOptions, whereOptions) {
        return this.resolveTable(table).getRowPosition(sortOptions, whereOptions);
    }
    /**
     * Get total row count in a table.
     * @param table The table to get row count from.
     * @param options The options for this query.
     * @returns An object with the count property.
     */
    getRowCount(table, options) {
        return this.resolveTable(table).getRowCount(options);
    }
    /**
     * Sets a row into the database (or updates existing one).
     * @param table The table to perform this action on.
     * @param data The data to set.
     * @param options Options for this query.
     * @returns Sqlite result for this setter.
     */
    set(table, data, options) {
        return this.resolveTable(table).set(data, options);
    }
    /**
     * Deletes rows from a database, if no options are given it will clean the entire table.
     * @param table The table to delete rows from.
     * @param options Options for this query.
     * @returns Affected rows and changes from the sqlite table.
     */
    delete(table, options) {
        return this.resolveTable(table).delete(options);
    }
    /**
     * Insert one or multiple rows into the database.
     * If a row with given primary key exists, this will throw an error.
     * @param table The table to set these rows to.
     * @param data The row or rows to insert.
     * @returns The sqlite changes happened in the table.
     */
    insert(table, data) {
        return this.resolveTable(table).insert(data);
    }
    /**
     * Gets a single row from the database.
     * @param table The table to get a row from.
     * @param options The options for this query.
     * @returns A row from the database, if it exists.
     * @example
     * db.get("mytable", {
     *      where: {
     *          column: "id",
     *          equals: "12345"
     *      }
     * })
     */
    get(table, options) {
        return this.resolveTable(table).get(options);
    }
    /**
     * Connects the database to its corresponding file.
     * @returns The connected database.
     */
    connect() {
        return new Promise((resolve) => {
            this.db = (0, better_sqlite3_1.default)(this.options.path);
            for (const table of this.arrayFromMap(this.tables)) {
                table.create();
            }
            this.readyAt = new Date();
            this.emit("ready");
            resolve(this);
        });
    }
    /**
     * Closes this database connection.
     * @returns
     */
    close() {
        var _a;
        (_a = this.db) === null || _a === void 0 ? void 0 : _a.close();
        this.readyAt = undefined;
        return true;
    }
}
exports.default = Database;
//# sourceMappingURL=Database.js.map