import { ColumnResolvable, DataResolvable, DataTypes, SortOptions, SqliteChanges, SqliteQueryOptions, SqliteTableColumn } from "../typings";
import { Column } from "./Column";
import Database from "./Database";
/**
 * Manages sqlite methods for a table.
 */
declare class Table {
    /**
     * The database for this table.
     */
    db: Database;
    /**
     * The name of this table.
     */
    name: string;
    /**
     * The raw unresolved columns for this table.
     */
    _columns: ColumnResolvable[];
    /**
     * The columns that were instantiated once connecting to database.
     */
    columns: Column[];
    /**
     * Whether the table is ready.
     */
    ready: boolean;
    /**
     *
     * @param name The name of the table.
     * @param db The database this table belongs to.
     * @param columns The columns to pre-define for this table
     */
    constructor(name: string, db: Database, columns?: ColumnResolvable[]);
    /**
     * Sets the database for this table.
     * @param db The database to attach this table to.
     * @returns The table in case.
     */
    setDatabase(db: Database): this;
    /**
     * Sets the name to this table.
     * @param str The new table name.
     * @returns The table in case.
     */
    setName(str: string): this;
    transformColumns(): Column[];
    getTypeColumns(type: DataTypes): Column[];
    getColumns(): Column[];
    getColumn(index: number): Column | undefined;
    isReady(): boolean;
    get columnCreator(): string;
    getRowCount(options?: SqliteQueryOptions): {
        count: number;
    };
    getTableColumns(): SqliteTableColumn[];
    create(): boolean;
    addColumns(columns: ColumnResolvable[], force?: boolean): this;
    addColumn(column: ColumnResolvable, force?: boolean): this;
    getRowPosition<K extends SqliteQueryOptions | undefined = undefined>(sortOptions: SortOptions, filterOptions: K): DataResolvable[] | {
        position: number;
    };
    get<K = undefined>(options: SqliteQueryOptions): K extends undefined ? DataResolvable : K;
    /**
     * Retrieves all or multiple results from this table.
     * @param options The options to use for filtering this query.
     * @returns The results from this table.
     */
    all<K = undefined>(options?: SqliteQueryOptions): K extends undefined ? DataResolvable[] : K[];
    /**
     * Inserts one or multiple rows into this table, if there is a primary key for this table and it already exists, this will throw an error.
     * @param data The row or rows to set into the table.
     * @returns Sqlite change report.
     */
    insert<K = undefined>(data: K extends undefined ? DataResolvable[] | DataResolvable : K): SqliteChanges;
    /**
     * Deletes a row or rows from this table, leave empty to clean the entire table.
     * @param options The options to filter this query.
     * @returns The sqlite changes report.
     */
    delete(options?: SqliteQueryOptions): SqliteChanges;
    /**
     * Sets a row into this table, or updates it if it already exists.
     * @param data The data for this row.
     * @param options Options to insert or update this row.
     * @returns Sqlite changes report, if any.
     */
    set<K = undefined>(data: K extends undefined ? DataResolvable : K, options?: SqliteQueryOptions): (SqliteChanges | undefined)[];
}
export { Table };
//# sourceMappingURL=Table.d.ts.map