"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Table = void 0;
const Util_1 = require("../util/Util");
const Column_1 = require("./Column");
const DatabaseError_1 = require("./DatabaseError");
/**
 * Manages sqlite methods for a table.
 */
class Table {
    /**
     *
     * @param name The name of the table.
     * @param db The database this table belongs to.
     * @param columns The columns to pre-define for this table
     */
    constructor(name, db, columns) {
        /**
         * The raw unresolved columns for this table.
         */
        this._columns = new Array();
        /**
         * The columns that were instantiated once connecting to database.
         */
        this.columns = new Array();
        /**
         * Whether the table is ready.
         */
        this.ready = false;
        this.db = db;
        this.name = name;
        if (columns) {
            this._columns = columns;
        }
    }
    /**
     * Sets the database for this table.
     * @param db The database to attach this table to.
     * @returns The table in case.
     */
    setDatabase(db) {
        this.db = db;
        return this;
    }
    /**
     * Sets the name to this table.
     * @param str The new table name.
     * @returns The table in case.
     */
    setName(str) {
        this.name = str;
        return this;
    }
    transformColumns() {
        return this._columns.map(c => Column_1.Column.transform(c, this.db));
    }
    getTypeColumns(type) {
        return this.columns.filter(c => c.type === type);
    }
    getColumns() {
        return this.columns;
    }
    getColumn(index) {
        return this.columns[index];
    }
    isReady() {
        return this.ready;
    }
    get columnCreator() {
        return this.columns.map(c => c.creator).join(", ");
    }
    getRowCount(options) {
        var _a;
        const opts = Util_1.Util.makeWhereOptions(options);
        const data = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`SELECT COUNT() as count FROM ${this.name} ${opts}`).get();
        return {
            count: data.count
        };
    }
    getTableColumns() {
        var _a;
        return (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.pragma(`table_info(${this.name})`);
    }
    create() {
        var _a;
        const got = this.db.getTableData(this.name);
        if (got) {
            const columns = this.getTableColumns();
            this.transformColumns().map(c => this.columns.push(c));
            for (const column of this.columns) {
                const clm = columns.find(c => c.name === column.name);
                if (!clm) {
                    column.create(this);
                }
            }
            this.ready = true;
            return false;
        }
        this.transformColumns().map(c => this.columns.push(c));
        const e = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`CREATE TABLE ${this.name}(${this.columnCreator})`).run();
        this.ready = true;
        return true;
    }
    addColumns(columns, force = false) {
        columns.map(c => this.addColumn(c, force));
        return this;
    }
    addColumn(column, force = false) {
        var _a;
        this._columns.push(column);
        if (force) {
            (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`ALTER TABLE ${this.name} ADD COLUMN ${column.name}`).run();
            this.columns.push(Column_1.Column.transform(column, this.db));
        }
        return this;
    }
    getRowPosition(sortOptions, filterOptions) {
        var _a, _b;
        const opts = Util_1.Util.makeSortOptions(sortOptions);
        const where = Util_1.Util.makeWhereOptions(filterOptions);
        const data = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`
            SELECT * FROM ( 
                SELECT
                ROW_NUMBER () OVER ( 
                ${opts} 
                ) 
                    RowNum,
                    ${(_b = filterOptions === null || filterOptions === void 0 ? void 0 : filterOptions.columns) !== null && _b !== void 0 ? _b : "*"}
                FROM
                    ${this.name}
                )
            ${where}
        `).all();
        if (!data) {
            throw new DatabaseError_1.DatabaseError(`Unknown data.`);
        }
        if (data.length !== 1) {
            return data;
        }
        return {
            position: data[0].RowNum
        };
    }
    get(options) {
        var _a, _b, _c;
        const opts = Util_1.Util.makeWhereOptions(options);
        const select = Util_1.Util.makeSelectOptions(options === null || options === void 0 ? void 0 : options.columns);
        const inner = Util_1.Util.makeInnerJoinOptions(options.innerJoin);
        const data = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`SELECT ${select.query} FROM ${this.name} ${inner} ${opts}`).get();
        if (data === undefined) {
            const defaultData = {};
            for (const column of this.columns) {
                defaultData[column.name] = (_b = column.default) !== null && _b !== void 0 ? _b : null;
            }
            if (options) {
                const whereOps = Array.isArray(options.where) ? options.where : [options.where];
                for (const opt of whereOps.filter(opt => (opt === null || opt === void 0 ? void 0 : opt.column) && (opt === null || opt === void 0 ? void 0 : opt.equals) !== undefined)) {
                    defaultData[opt === null || opt === void 0 ? void 0 : opt.column] = (_c = opt === null || opt === void 0 ? void 0 : opt.equals) !== null && _c !== void 0 ? _c : null;
                }
            }
            return defaultData;
        }
        if (this.db.options.sanitize) {
            for (const column of this.getTypeColumns("JSON")) {
                const name = select.renamedColumns[column.name] ? select.renamedColumns[column.name] : column.name;
                if (data[name] !== null && data[name] !== undefined) {
                    data[name] = JSON.parse(data[name]);
                }
            }
            for (const column of this.getTypeColumns("BOOLEAN")) {
                const name = select.renamedColumns[column.name] ? select.renamedColumns[column.name] : column.name;
                if (data[name] !== null && data[name] !== undefined) {
                    data[name] = data[name] === 1;
                }
            }
        }
        return data;
    }
    /**
     * Retrieves all or multiple results from this table.
     * @param options The options to use for filtering this query.
     * @returns The results from this table.
     */
    all(options) {
        var _a;
        const opts = Util_1.Util.makeWhereOptions(options);
        const inner = Util_1.Util.makeInnerJoinOptions(options === null || options === void 0 ? void 0 : options.innerJoin);
        const select = Util_1.Util.makeSelectOptions(options === null || options === void 0 ? void 0 : options.columns);
        const data = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`SELECT ${select.query} FROM ${this.name} ${inner} ${opts}`).all();
        if (this.db.options.sanitize && this.db.options.length !== undefined) {
            if (this.db.options.length > 0 && data.length <= this.db.options.length) {
                for (let i = 0; i < data.length; i++) {
                    const d = data[i];
                    for (const column of this.getTypeColumns("JSON")) {
                        const name = select.renamedColumns[column.name] ? select.renamedColumns[column.name] : column.name;
                        if (d[name] !== null && d[name] !== undefined) {
                            d[name] = JSON.parse(d[name]);
                        }
                    }
                    for (const column of this.getTypeColumns("BOOLEAN")) {
                        const name = select.renamedColumns[column.name] ? select.renamedColumns[column.name] : column.name;
                        if (d[name] !== null && d[name] !== undefined) {
                            d[name] = d[name] === 1;
                        }
                    }
                }
            }
        }
        return data;
    }
    /**
     * Inserts one or multiple rows into this table, if there is a primary key for this table and it already exists, this will throw an error.
     * @param data The row or rows to set into the table.
     * @returns Sqlite change report.
     */
    insert(data) {
        var _a;
        const datas = Array.isArray(data) ? data : [data];
        const columns = Util_1.Util.makeColumnsFrom(datas[0]);
        const values = datas.map(d => `(${Util_1.Util.makeInsertValues(d, this.columns)})`);
        return (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`INSERT INTO ${this.name}(${columns}) VALUES ${values.join(", \n")}`).run();
    }
    /**
     * Deletes a row or rows from this table, leave empty to clean the entire table.
     * @param options The options to filter this query.
     * @returns The sqlite changes report.
     */
    delete(options) {
        var _a;
        const opts = Util_1.Util.makeWhereOptions(options);
        return (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`DELETE FROM ${this.name} ${opts}`).run();
    }
    /**
     * Sets a row into this table, or updates it if it already exists.
     * @param data The data for this row.
     * @param options Options to insert or update this row.
     * @returns Sqlite changes report, if any.
     */
    set(data, options) {
        var _a, _b;
        const opts = Util_1.Util.makeWhereOptions(options);
        const values = Util_1.Util.makeSetValues(data, this.columns);
        const insert = Util_1.Util.makeInsertValues(data, this.columns);
        const columns = Util_1.Util.makeColumnsFrom(data);
        const ch1 = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`INSERT OR IGNORE INTO ${this.name} (${columns}) VALUES (${insert})`).run();
        const ch2 = (_b = this.db.db) === null || _b === void 0 ? void 0 : _b.prepare(`UPDATE ${this.name} SET ${values} ${opts}`).run();
        return [
            ch1,
            ch2
        ];
    }
}
exports.Table = Table;
//# sourceMappingURL=Table.js.map