import { Column } from "../structures/Column";
import { Table } from "../structures/Table";
export declare type If<A extends boolean, K, R = null> = A extends true ? K : A extends false ? R : R;
/**
 * Available options for the database.
 */
export interface DatabaseOptions {
    /**
     * The path to the database file.
     * @default ./db.sqlite
     */
    path?: string;
    /**
     * Whether to convert boolean and json into their real values and not leave them as integers or strings.
     * @default true
     */
    sanitize?: boolean;
    /**
     * Max length a query result can have to stop sanitizing the ouput.
     * @default 1
     */
    length?: number;
}
/**
 * All events fired by this database.
 */
export interface DatabaseEvents {
    /**
     * Fired whenever the database becomes ready.
     */
    ready: () => void;
    expire: (data: DataResolvable) => void;
    /**
     * Fired whenever the database is told to set data.
     */
    dataSet: (id: string, old: Record<string, any> | null, data: Record<string, any>) => void;
    /**
     * Fired whenver the database is called to get data.
     */
    dataGet: (id: string, data: Record<string, any> | undefined) => void;
    /**
     * Fired whenever the database is called to delete data.
     */
    dataDelete: (id: string) => void;
}
/**
 * Options to use for a condition.
 */
export interface WhereColumnOptions {
    /**
     * The column to apply this condition on.
     */
    column: string;
    /**
     * Whether the column must be equal to a number or string.
     */
    equals?: string | number;
    /**
     * Whether the column must be greater than a number.
     */
    greater?: number;
    /**
     * Whether the column has to be lesser than a number.
     */
    lesser?: number;
    /**
     * Whether the column has to be lesser than or equal to a number.
     */
    lesserEquals?: number;
    /**
     * Whether the column has to be greater than or equal to a number.
     */
    greaterEquals?: number;
    /**
     * Whether the column value must be included in one of the given values.
     */
    in?: (number | string)[];
    /**
     * Whether the column row value starts with a string.
     */
    startsWith?: string;
    /**
     * Whether the column row value ends with a string
     */
    endsWith?: string;
    /**
     * Whether the column row value is not equal to a number or string
     */
    notEqual?: number | string;
    /**
     * Whether the column row value includes a string.
     */
    includes?: string;
    /**
     * Whether to make this a OR condition.
     */
    or?: boolean;
    /**
     * Whether to check the opposite for this condition.
     */
    not?: boolean;
}
/**
 * Data for an object.
 */
export declare type DataResolvable = Record<string, any>;
/**
 * Changes happened in the sqlite database.
 */
export interface SqliteChanges {
    /**
     * Affected row count.
     */
    changes: number;
    /**
     * Last row id that was modified.
     */
    lastInsertRowid: number;
}
/**
 * Options to sort the query by.
 */
export interface SortOptions {
    /**
     * The column or columns to sort by.
     */
    column: string | string[];
    /**
     * The order to sort by.
     * @default DESC
     */
    order?: "ASC" | "DESC";
    /**
     * Operator to use to sort.
     * @default +
     */
    operator?: "+" | "/" | "*" | "%" | "-";
}
export interface ColumnSelectOptions {
    name: string;
    as?: string;
}
/**
 * Options to join a query with another table.
 */
export interface InnerJoinOptions {
    /**
     * Table to join with.
     */
    table: string;
    /**
     * Table we are joining from.
     */
    from: string;
    /**
     * Column to use to join these results.
     */
    column: string;
}
export interface RemoteDatabaseOptions extends DatabaseOptions {
    port: number;
    password?: string;
    host: string;
}
/**
 * The options for a sqlite query.
 */
export interface SqliteQueryOptions {
    /**
     * Columns to get from the database, * stands for all.
     * @default *
     */
    columns?: string[] | "*" | ColumnSelectOptions[];
    /**
     * Options to retrieve data from other tables.
     */
    innerJoin?: InnerJoinOptions[];
    /**
     * Options to filter the query by.
     */
    where?: WhereColumnOptions | WhereColumnOptions[];
    /**
     * The limit of results for this query.
     */
    limit?: number;
    /**
     * Position to start counting results.
     * @default 0
     */
    offset?: number;
    /**
     * Sort options for this query.
     */
    sort?: SortOptions;
}
/**
 * The options for a table column.
 */
export interface ColumnOptions {
    /**
     * The name of this column.
     */
    name: string | null;
    /**
     * The default value for this column.
     */
    default?: any | null;
    /**
     * Whether this column can be nullable in the database.
     */
    notNull?: boolean | null;
    /**
     * Whether this column is the primary key to the table.
     */
    primary?: boolean | null;
    /**
     * The type of this column.
     */
    type: DataTypes | null;
}
/**
 * Object containing data from a sqlite column.
 */
export interface SqliteTableColumn {
    /**
     * The column id.
     */
    cid: number;
    /**
     * The name of this column.
     */
    name: string;
    /**
     * The type of this column.
     */
    type: SqliteDataTypes;
    /**
     * Whether this column can be nullable.
     */
    notnull: number;
    /**
     * The default value for this column.
     */
    dflt_value: number | null | string;
    /**
     * Whether this column is the primary key to table.
     */
    pk: number;
}
/**
 * A string with the column name or a column class / object.
 */
export declare type ColumnResolvable = Column | ColumnOptions;
/**
 * Holds valid data types for sqlite databases.
 */
export declare type SqliteDataTypes = "INTEGER" | "NUMERIC" | "BLOB" | "TEXT" | "REAL";
/**
 * Holds valid data types accepted by the database.
 */
export declare type DataTypes = "INTEGER" | "TEXT" | "NUMERIC" | "BLOB" | "JSON" | "REAL" | "BOOLEAN";
/**
 * A string with the table name or a table object / class.
 */
export declare type TableResolvable = string | Table | {
    name: string;
};
//# sourceMappingURL=index.d.ts.map