"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Util = void 0;
const Column_1 = require("../structures/Column");
/**
 * Contains utility functions to make interacting with the database easier.
 */
class Util extends null {
    constructor() { }
    static mergeDefault(target, source) {
        for (const [key, value] of Object.entries(source)) {
            const current = target[key];
            const def = source[key];
            if (current === undefined || current === null) {
                target[key] = def;
            }
            else if (!Array.isArray(def) && typeof def === "object") {
                Util.mergeDefault(current, def);
            }
        }
        return target;
    }
    /**
     * Creates inner join options for a query.
     * @param options The options for joining tables.
     * @returns The created inner join query.
     */
    static makeInnerJoinOptions(options) {
        if (!options)
            return "";
        const opts = [];
        for (const opt of options) {
            opts.push(`INNER JOIN ${opt.table} ON ${opt.table}.${opt.column} = ${opt.from}.${opt.column}`);
        }
        return opts.join(" \n");
    }
    /**
     * Creates select options for a query.
     * @param options The options for this query selector.
     * @returns The created selector.
     */
    static makeSelectOptions(options) {
        const renamedColumns = {};
        if (!options || options === "*")
            return {
                renamedColumns,
                query: "*"
            };
        const opts = [];
        for (const opt of options) {
            if (typeof opt === "object" && opt.as) {
                renamedColumns[opt.name] = opt.as;
            }
            opts.push(typeof opt === "string" ?
                opt :
                opt.as ?
                    `${opt.name} AS ${opt.as}` :
                    opt.name);
        }
        return {
            query: opts.join(", \n"),
            renamedColumns
        };
    }
    /**
     * Makes filters for a query.
     * @param options The options for this filter.
     * @returns The returned filter for a query.
     */
    static makeWhereOptions(options) {
        var _a, _b;
        if (!options)
            return "";
        const opts = ["WHERE"];
        if (options.where) {
            const ops = Array.isArray(options.where) ? options.where : [options.where];
            let i = 0;
            for (const where of ops) {
                if (where.endsWith !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} LIKE '%${where.endsWith}'`);
                }
                else if (where.startsWith !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} LIKE '${where.startsWith}%'`);
                }
                else if (where.lesser !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} < ${where.lesser}`);
                }
                else if (where.greater !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} > ${where.greater}`);
                }
                else if (where.equals !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} = ${typeof where.equals === "string" ? `'${where.equals}'` : where.equals}`);
                }
                else if (where.in !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} IN (${where.in.map(n => typeof n === "string" ? `'${n}'` : n).join(", ")})`);
                }
                else if (where.includes !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} LIKE '%${where.includes}%'`);
                }
                else if (where.lesserEquals !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} <= ${where.lesserEquals}`);
                }
                else if (where.greaterEquals !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} >= ${where.greaterEquals}`);
                }
                else if (where.notEqual !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} != ${typeof where.notEqual === "string" ? `'${where.notEqual}'` : where.notEqual}`);
                }
                i++;
                if (ops[i]) {
                    if (where.or) {
                        opts.push(`OR`);
                    }
                    else {
                        opts.push(`AND`);
                    }
                }
            }
        }
        if (options.sort) {
            const columns = Array.isArray(options.sort.column) ? options.sort.column : [options.sort.column];
            opts.push(`ORDER BY (${columns.join((_a = options.sort.operator) !== null && _a !== void 0 ? _a : "+")}) ${(_b = options.sort.order) !== null && _b !== void 0 ? _b : "DESC"}`);
        }
        if (options.limit) {
            opts.push(`LIMIT ${options.limit}`);
        }
        if (options.offset) {
            opts.push(`OFFSET ${options.offset}`);
        }
        return opts.join(" \n");
    }
    /**
     * Creates columns for a table.
     * @param data The object for columns.
     * @returns The columns that were created from the object.
     */
    static makeColumnsFrom(data) {
        return Object.keys(data).join(", ");
    }
    /**
     * Creates a string of sort options for this query.
     * @param options The options for sorting the query.
     * @returns The string to sort this query.
     */
    static makeSortOptions(options) {
        var _a, _b;
        if (!options)
            return "";
        const total = [];
        const columns = Array.isArray(options.column) ? options.column : [options.column];
        total.push(`ORDER BY (${columns.join((_a = options.operator) !== null && _a !== void 0 ? _a : "+")}) ${(_b = options.order) !== null && _b !== void 0 ? _b : "DESC"}`);
        return total.join(" \n");
    }
    /**
     * Makes a string that is valid to insert values into sqlite.
     * @param data The object containing data.
     * @param columns The columns of the table.
     * @returns The string to use against sqlite.
     */
    static makeInsertValues(data, columns) {
        const got = [];
        for (const [key, val] of Object.entries(data)) {
            const column = columns.find(c => c.name === key);
            if (!column) {
                continue;
            }
            const g = Column_1.Column.transformValue(val, column.type);
            got.push(g === null ? `${g}` : g);
        }
        return got.join(", ");
    }
    /**
     * Creates a set of values to update a row.
     * @param data The object data to update with.
     * @param columns The columns of the table.
     * @returns The string that can be used to update this row.
     */
    static makeSetValues(data, columns) {
        const got = [];
        for (const [key, val] of Object.entries(data)) {
            const column = columns.find(c => c.name === key);
            if (!column) {
                continue;
            }
            got.push(`${key} = ${Column_1.Column.transformValue(val, column.type)}`);
        }
        return got.join(", \n");
    }
    /**
     * Checks whether a number is valid for sqlite.
     * @param n The number to check.
     * @returns Whether the number is valid.
     */
    static isNumber(n) {
        return !isNaN(Number(n)) && n <= Number.MAX_SAFE_INTEGER && n >= Number.MIN_SAFE_INTEGER;
    }
}
exports.Util = Util;
//# sourceMappingURL=Util.js.map